@@ -19,12 +19,11 @@ async def main():
     params: dict[str, Any]  = {
         'api_key': os.environ['MOLANO_PICQER_API_KEY'],
         'api_email': 'test@headless.python.dev.unimatrixone.io',
-        'api_url': 'https://molano.picqer.com/api',
-        'recover_ratelimit': True
+        'api_url': 'https://molano.picqer.com/api'
     }
     async with Client(**params) as client:
         requests: list[Awaitable[User]] = []
-        for _ in range(512):
+        for _ in range(600):
             requests.append(client.retrieve(User, 13631))
         await asyncio.gather(*requests)
 
