@@ -21,6 +21,7 @@ import pydantic
 from .headers import Headers
 from .ibackoff import IBackoff
 from .icredential import ICredential
+from .ratelimited import RateLimited
 from .iresource import IResource
 from .iresponse import IResponse
 from .irequest import IRequest
@@ -82,7 +83,12 @@ class IClient(Generic[Request, Response]):
             json=json
         )
         await (credential or self.credential).add_to_request(request)
-        response = await self.send(request)
+        try:
+            response = await self.send(request)
+        except Exception as exc:
+            response = await request.on_failure(exc, self)
+            if response is None:
+                raise
         if response.status_code == 429:
             response = await self.on_rate_limited(response)
         return response
@@ -108,7 +114,14 @@ class IClient(Generic[Request, Response]):
         it is rate limited. The default implementation raises an exception, but
         subclasses may override this method to return a response object.
         """
-        return await self.backoff.retry(self, response.request, response)
+        response = await self.backoff.retry(self, response.request, response)
+        if response.status_code == 429:
+            self.logger.critical(
+                "Unable to recover from rate limit (request: %s, resource: %s)",
+                response.request.id, response.request.url
+            )
+            raise RateLimited(request=response.request, response=response)
+        return response
 
     def process_response(self, action: str, data: dict[str, Any] | list[Any]) -> dict[str, Any]:
         """Hook to transform response data."""
